//-----------------------------------------------------------------------------
// AS_Format.h
// Copyright (c) Sonic Solutions.  All rights reserved.
//-----------------------------------------------------------------------------

///////////////////////////////////////////////////////////////////////////////
// PPG Notes
// Always create VIDEO_TS / HDDVD_TS folders, then add them to a corresponding Layout's volume
//
// AS_Format allows the creation of known logical formats destined for a variety of targets
//
// Who   Date     What
// GK 02-14-05 Initial
///////////////////////////////////////////////////////////////////////////////

#ifndef _AS_FormatH
#define _AS_FormatH

#include "AS_StorageError.h"
#include "AS_StorageDevice.h"
#include "AS_Volume.h"

typedef AS_String AS_URL;
typedef AS_String AS_PlanProject;      // XML Project File (Unicode)

struct AS_Format : public AS_Volume
{
	// Properties of AS_Format Objects
	enum Property
	{
		Prop_Project          = 1,        // NOT IMPLEMENTED: Flattened Layout Object as XML Unicode Text
		Prop_SourceVolume     = 2,        // IN/OUT: AS_Volume specifies source volume
		Prop_VideoFreeBlocks  = 4,        // OUT: free space suitable for video (meets rules for alignment and minimum extent size)

		// Multi-Layer Properties
		Prop_Layer_Base       = 0x0100,
		Prop_LayerBlocks      = Prop_Layer_Base + 1, // OUT: UInt32 For multi-layer formats, the number of
													 //      blocks in a layer (for multi-layer only after _Prepare())
													 // propertyIndex = layer number
		Prop_LogicalLayerBrk  = Prop_Layer_Base + 2, // IN: LogicalLayerBreak struct, specifies the byte offset
													 //      into a file where the LB should be
													 //      Call AS_Format_GetLayerBreakRange() to get range of
													 //      valid LogicalLayerBreaks

		// CD-DA Specific Properties
		Prop_Audio_Base       = 0x0200,
		// CD-Text strings are of type wchar_t unless otherwise specified.
		Prop_Audio_MediaCatalogNum = Prop_Audio_Base + 1,  // IN/OUT: String CD-Text Media Catalog Number
		Prop_Audio_Title           = Prop_Audio_Base + 2,  // IN/OUT: String CD-Text Title
		Prop_Audio_Performer       = Prop_Audio_Base + 3,  // IN/OUT: String CD-Text Performer
		Prop_Audio_Songwriter      = Prop_Audio_Base + 4,  // IN/OUT: String CD-Text Songwriter
		Prop_Audio_Composer        = Prop_Audio_Base + 5,  // IN/OUT: String CD-Text Composer
		Prop_Audio_Arranger        = Prop_Audio_Base + 6,  // IN/OUT: String CD-Text Arranger
		Prop_Audio_Message         = Prop_Audio_Base + 7,  // IN/OUT: String CD-Text Message
		Prop_Audio_Genre           = Prop_Audio_Base + 8,  // IN/OUT: String CD-Text Genre
		Prop_Audio_Genre_Code      = Prop_Audio_Base + 9,  // IN/OUT: String CD-Text Genre Code
		Prop_Audio_NumLanguages    = Prop_Audio_Base + 10, // OUT: UInt32 Number of languages with CD-Text
		Prop_Audio_Languages       = Prop_Audio_Base + 11, // OUT: UInt8[] Array of CD-Text language codes. See CDTextLanguageType below.

		// VCD Specific Properties (reserved for future properties)
		Prop_VCD_Base         = 0x0300,

		// DVD-V Specific Properties (reserved for future properties)
		Prop_DVD_Base				= 0x0400,
		Prop_DVD_PlaybackRegion     = Prop_DVD_Base + 1,	// IN/OUT: UInt8 specifying regions bits which are dis-allowed
		Prop_DVD_EccAligned			= Prop_DVD_Base + 2,	// IN: UInt32 forces ECC alignment of VOB files
															//    0 = do not align VOB files
															//    1 = align VOB files on ECC block boundaries
															// Note: VOB files always aligned for QFlix consumer and DVD-Download media.
															//       In all other cases, does not do alignment by default.

		// CSS Specific Properties
		Prop_CSS_Base         = 0x0500,     // For DDP2 Targets ( CPR_MAI)
		Prop_CSS_KeySet       = Prop_CSS_Base + 1,  // IN: CSSKeyStruct for setting CSS keys
		Prop_CSS_TitleKeys    = Prop_CSS_Base + 2,  // IN: TitleKeySet for setting keys per title set

		// AACS Specific Properties
		Prop_AACS_Base       = 0x0600,      // For Type_DDP30 and Type_CMF20_HD Targets
		Prop_AACS_VolumeID   = Prop_AACS_Base + 1, // IN: AS_String containing volume ID (1 per volume)
		Prop_AACS_TitleKey   = Prop_AACS_Base + 2, // IN: AS_String containing title key (may add 1 or more per volume)
		Prop_AACS_StartPMSN  = Prop_AACS_Base + 3, // IN: AS_String containing start of PMSN range
		Prop_AACS_EndPMSN    = Prop_AACS_Base + 4, // IN: AS_String containing end of PMSN range

		// DDP/CMF General Properties
		Prop_DDP_Base         = 0x0700,
		Prop_DDP_MasterID    = Prop_DDP_Base + 1, // IN: In as AS_String containing ASCII characters,
		Prop_DDP_NumLayers   = Prop_DDP_Base + 2, // IN/OUT: UInt32 Number of layers on final disc
		Prop_DDP_NumSides    = Prop_DDP_Base + 3, // IN/OUT: UInt32 Number of sides on final disc (1 or 2)
		Prop_DDP_CurrentSide = Prop_DDP_Base + 4, // IN/OUT: UInt32 Current side of final disc (1 or 2)
		Prop_DDP_TrackPath   = Prop_DDP_Base + 5, // IN/OUT: UInt32 Direction of translation (only valid on dual-layer)
												  //    0 = Opposite Track Path DVD-ROM Being requested
												  //    1 = Parallel Track Path DVD-ROM being requested
		Prop_DDP_DiscSize    = Prop_DDP_Base + 6, // IN/OUT: UInt32 Size in cm of disc to be mastered (8 or 12)
		Prop_DDP_StorageMode = Prop_DDP_Base + 7, // IN/OUT: UInt32 Source Storage Mode
												  //    0 = User data only, 2048 bytes
												  //    1 = Complete 2054 bytes
												  //    6 = (currently not supported) Incomplete 2064 bytes
												  //    7 = (currently not supported) Complete 2064 bytes
		Prop_DDP_Span_AS_StorageDevice = Prop_DDP_Base + 8, // IN: AS_StorageDevice to span to 2nd DLT device for 2nd layer
		Prop_DDP_ChecksumFile = Prop_DDP_Base + 9,// IN/OUT: UInt32 indicates whether to write ImageIntegrity Checksum File
												  //    0 = Do not write checksum file
												  //    1 = Write checksum file (default)
												  //    NOTE: Prop_DDP_ChecksumFile and Prop_DDP_DiscTag are mutually exclusive.
												  //          If both are set, the last one set will be used.
		Prop_DDP_InputImage = Prop_DDP_Base + 10, // IN: AS_String containing full path of existing image file to be used in DDP(or CMF) file set
												  //     This is optional and is only to be used when an image already exists, that the client
												  //     wants to create a DDP (or CMF) set from. (image assumed to be only user data 2048 bytes per sector)
		Prop_DDP_DiscTag    = Prop_DDP_Base + 11, // CURRENTLY NOT SUPPORTED - RESERVED FOR FUTURE USE
												  // IN: DiscTagProperties structure indicates that DiscTag should be written
												  //     and supplies the DiscTag information.
												  //    NOTE: Prop_DDP_ChecksumFile and Prop_DDP_DiscTag are mutually exclusive.
												  //          If both are set, the last one set will be used.
		Prop_DDP_Signature = Prop_DDP_Base + 12,  // IN: UInt32 indicates whether to write Image Signature file
												  //    0 = Do not write signature file
												  //    1 = Write signature file (default)
												  //    NOTE: Prop_DDP_Signature and Prop_DDP_DiscTag are mutually exclusive.
												  //          If both are set, the last one set will be used.

		// DDP2 Specific Properties
		Prop_DDP2_Base        = 0x0800,
		Prop_DDP2_SscrStatus  = Prop_DDP2_Base + 1, // IN/OUT: UInt32 Security Scrambling Status
													//    0 = The data for the disc contains no security scrambling. The final disc will not be scrambled for security.
													//    1 = The data for the disc contains no security scrambling. The final disc will be scrambled for security.
													//    2 = (currently not supported) The data for the disc contains security scrambling. The final disc will be scrambled for security.
													//    3 = The data for the disc contains no security scrambling. The security scrambling keys are read from a floppy disk. Format the floppy according to CSS Site Guide Book. The final disc will be scrambled for security.

		// DDP3 Specific Properties (also applies to HD CMF)
		Prop_DDP3_Base             = 0x0900,      // DDP3 Properties
		Prop_DDP3_Title            = Prop_DDP3_Base + 1,    // IN: AS_String containing the title of the disc image contents
		Prop_DDP3_Author           = Prop_DDP3_Base + 2,    // IN: AS_String containing the name and version of the authoring system used to create the disc image
		Prop_DDP3_CopyrightNotice  = Prop_DDP3_Base + 3,    // IN: AS_String containing the copyright notice for the disc image contents
		Prop_DDP3_Abstract         = Prop_DDP3_Base + 4,    // IN: AS_String containing a description of the disc image contents
		Prop_DDP3_DiscID           = Prop_DDP3_Base + 5,    // IN: AS_String containing a GUID to uniquely identify the disc
		Prop_DDP3_MediaType        = Prop_DDP3_Base + 6,    // IN: UInt32 indicates the type of disc being described by the DDP file set
															//    0 = HD DVD-ROM (default)
															//    1 = HD DVD-ROM Twin Format - indicates that layer 0 will contain a Standard Density DVD image
															//        and current DDP set is single layer that will go on layer 1 of destination disc
															//    2 = 3X-SPEED DVD-ROM
		Prop_DDP3_CopyProtectionInfo = Prop_DDP3_Base + 7,  // DEPRECATED

		// BD CMF Specific Properties
		Prop_BDCMF_Base               = 0x0A00,               // BD CMF Properties
		Prop_BDCMF_BD_Type            = Prop_BDCMF_Base + 1,  // IN: UInt32 indicates BD type
															  //    0 = BD (default)
															  //    1 = BD9 (BD content on DL DVD-ROM)
		Prop_BDCMF_HybridType         = Prop_BDCMF_Base + 2,  // IN: UInt32 indicates hybrid type
															  //    0 = no hybrid (default)
															  //    1 = BD with CD hybrid
															  //    2 = BD with DVD hybrid
		Prop_BDCMF_ContentCategory    = Prop_BDCMF_Base + 3,  // IN: UInt32 indicates category type
															  //    0 = BD-ROM Movie Media (default)
															  //    1 = BD-ROM Movie Media with BD Data
															  //    2 = BD-ROM Data Media
		Prop_BDCMF_AuthorsNote        = Prop_BDCMF_Base + 4,  // IN: AS_String containing an optional note from Authoring Studio to Replication Facility (4096 characters max)
		Prop_BDCMF_BDI_Usage          = Prop_BDCMF_Base + 5,  // IN: UInt32 indicating usage of Basic Disc Information
															  //    0 = None (default)
															  //    1 = Use Basic Disc Information
		Prop_BDCMF_PMSN_Usage         = Prop_BDCMF_Base + 6,  // IN: UInt32 indicating usage of Pre-recorded Media Serial Number (PMSN)
															  //    0 = None (default)
															  //    1 = Use conventional PMSN
															  //    2 = Use Un-Guessable PMSN
		Prop_BDCMF_BCAID_SN_Usage     = Prop_BDCMF_Base + 7,  // IN: UInt32 indicating usage of BCAID for Serial Number and data length
															  //    0 = None (default)
															  //    1 = USE16
															  //    2 = USE32
		Prop_BDCMF_BCAID_UD_Usage     = Prop_BDCMF_Base + 8,  // IN: UInt32 indicating usage of BCAID for User Data and data length
															  //    0 = None (default)
															  //    1 = USE16
															  //    2 = USE32
		Prop_BDCMF_BCA_Usage          = Prop_BDCMF_Base + 9,  // IN: UInt32 indicating usage of BCA on BD9 media
															  //    0 = None (default)
															  //    1 = Use BCA
		Prop_BDCMF_DiscInfoNote       = Prop_BDCMF_Base + 10, // IN: AS_String containing any additional disc related information desired (4096 characters max)
		Prop_BDCMF_TitleName          = Prop_BDCMF_Base + 11, // IN: AS_String containing the title of the disc, such as movie title (256 characters max)
		Prop_BDCMF_CopyrightNotice    = Prop_BDCMF_Base + 12, // IN: AS_String containing the copyright notice
		Prop_BDCMF_BD_J_Usage         = Prop_BDCMF_Base + 13, // IN: UInt32 indicates whether content on the media uses BD-J
															  //    0 = does not use BD-J (default)
															  //    1 = uses BD-J
		Prop_BDCMF_BD_Live_Usage      = Prop_BDCMF_Base + 14, // IN: UInt32 indicates whether content on the media uses BD-Live
															  //    0 = does not use BD-Live (default)
															  //    1 = uses BD-Live
		Prop_BDCMF_Region             = Prop_BDCMF_Base + 15, // IN: UInt32 indicates region control status
															  //    0 = Allowed (default) - all of the disc can be played back
															  //    1 = Limited - a part of the disc can be played back
															  //    2 = Prohibited - entire disc can't be played back
															  //    3 = Not applicable - authoring studio can't set Region
															  // PropertyIndex parameter indicates which Region
															  //    0 = Region A (default)
															  //    1 = Region B
															  //    2 = Region C
															  // These regions must all be set or none. By setting one of them (i.e., A),
															  // the others (i.e., B and C) will all be all be set to the default (Allowed)
															  // until set explicitly.
		Prop_BDCMF_ContentOwnerNote   = Prop_BDCMF_Base + 16, // IN: AS_String containing any optional information from Content Owner (4096 characters max)
		Prop_BDCMF_TitleInfoNote      = Prop_BDCMF_Base + 17, // IN: AS_String containing any optional Title related information. Authoring Studio can use 
															  //     this space for a message to Disc Replication Facility (4096 characters max)
		Prop_BDCMF_AACS_Usage         = Prop_BDCMF_Base + 18, // IN: UInt32 indicates AACS usage on a media
															  //    0 = AACS is not used on media
															  //    1 = AACS is used on media (default)
		Prop_BDCMF_BDPlus_Usage       = Prop_BDCMF_Base + 19, // IN: UInt32 indicates BD+ usage on a media
															  //    0 = BD+ is not used on media (default)
															  //    1 = BD+ is used on media
		Prop_BDCMF_SequenceKey_Usage  = Prop_BDCMF_Base + 20, // IN: UInt32 indicates usage of Sequence Key
															  //    0 = Sequence Key not used (default)
															  //    1 = Sequence Key is used
		Prop_BDCMF_SRM_DTCP_Usage     = Prop_BDCMF_Base + 21, // IN: UInt32 indicates usage of SRM for DTCP
															  //    0 = SRM for DTCP not used (default)
															  //    1 = SRM for DTCP is used
		Prop_BDCMF_SRM_HDCP_Usage     = Prop_BDCMF_Base + 22, // IN: UInt32 indicates usage of SRM for HDCP
															  //    0 = SRM for HDCP not used (default)
															  //    1 = SRM for HDCP is used
		Prop_BDCMF_FacilityName       = Prop_BDCMF_Base + 23, // IN: AS_String containing the name of the facility where the BDCMF is generated (256 characters max)
		Prop_BDCMF_FacilityLocation   = Prop_BDCMF_Base + 24, // IN: AS_String containing the location of the facility where the BDCMF is generated (256 characters max)
		Prop_BDCMF_WorkID             = Prop_BDCMF_Base + 25, // IN: AS_String containing optional work ID of the project (256 characters max)
		Prop_BDCMF_Operator           = Prop_BDCMF_Base + 26, // IN: AS_String containing optional name of the operator performing the process (256 characters max)
		Prop_BDCMF_ToolName           = Prop_BDCMF_Base + 27, // IN: AS_String containing optional name of the tool performing the process (256 characters max)
		Prop_BDCMF_ToolVersion        = Prop_BDCMF_Base + 28, // IN: AS_String containing optional version of the tool performing the process (256 characters max)
		Prop_BDCMF_ProcessNote        = Prop_BDCMF_Base + 29, // IN: AS_String containing any additional process information desired (4096 characters max)
		Prop_BDCMF_PlayingTimeInfo    = Prop_BDCMF_Base + 30, // IN: PlayingTimeInfo struct containing the playing time of each codec used 
															  //     (this property must be called once for each codec)
		Prop_BDCMF_CpsUnitInfo        = Prop_BDCMF_Base + 31, // IN: CpsUnitInfo struct containing filename, encryption usage and CPS unit number for each m2ts file
															  //     If not set then defaults to all m2ts files set to be "Encrypted" and assigned to CPS unit number 1

		// DVD+VR Specific Properties
		Prop_PlusVR_Base              = 0x0B00,               // DVD+VR Properties
		Prop_PlusVR_RemappedFileLBA   = Prop_PlusVR_Base + 1, // OUT: UInt32 indicates sector address of where file is remapped to on an unfinalized DVD+VR disc
															  //     Parameters to AS_Format_GetProperty() shall be set as follows:
															  //     PropertyIndex - set to zero, ignored
															  //     extbufLen     - sizeof(AS_String)
															  //     extPropData   - pointer to AS_String containing the full path of the file
															  //     extpropSize   - NULL, ignored
															  //     
															  //     Should only be called after AS_Volume_Prepare() and before AS_Volume_Flush().
															  //     Used with unfinalized DVD+VR format written to DVD+R, for client to use 
															  //     to generate IFO file data.

		// MovieDisc Specific Properties
		Prop_MD_Base                  = 0x1000,               // MovieDisc Properties
		Prop_MD_TrailerReplaceInfo    = Prop_MD_Base + 1,     // IN/OUT: TrailerReplaceInfo struct, specifies source path, and optional index and additional source folder path for title (trailer) replacement
		Prop_MD_TargetStartSector     = Prop_MD_Base + 2,     // IN/OUT: UInt32 specifying logical block offset to move layout on target
		Prop_MD_Protection_Type       = Prop_MD_Base + 3,     // IN: MDProtectionTypes bitfield specifying copy protection type for targets
		Prop_MD_SourceProtectionIndex = Prop_MD_Base + 4,     // IN: UInt32 specifying source protection index
		Prop_MD_SourceAESKey          = Prop_MD_Base + 5,     // IN: specify 128 bits AES key which source is encrypted with (VOBs only).
		Prop_MD_SourceProtection_Type = Prop_MD_Base + 6,     // NOT IMPLEMENTED: MDSrcProtectionTypes bitfield specifying copy protection type for sources
	};

	// Volume state returned from GetProperty
	typedef UInt32 State;
	static const State IsDirty        = (1 << 0);
	static const State IsAppendable   = (1 << 1);

	// Logical Layer Break passed to SetProperty(Prop_LogicalLayerBrk)
	// specifies a file and byte offset into file where the layer break will be
	struct LogicalLayerBreak
	{
		AS_String fileName;  // file on destination that will span the layer break
		UInt64 byteOffset;   // exact byte offset into file where layer break will be
	};

	//  Title Key set
	struct TitleKeySet
	{
		SInt32 titleIndex;
		UInt8 keyInfo[5];
		UInt8 uKeyInfo[5];
	};

	// TrailerReplaceInfo passed to SetProperty(Prop_MD_TrailerReplaceInfo)
	// specifies a specific VTS title set (trailer) and the source folder path of the replacement trailer set.
	// If no replacement is needed, the value of replaceIndex = 0.
	// The value of replaceIndex = -1 for last title, -2 for 2nd from last, etc.
	struct TrailerReplaceInfo
	{
		SInt32 replaceIndex;
		AS_String replacementFolder;
	};

	//  Title Key set
	struct CSSKeyStruct
	{
		UInt8 ACC; // lower 5 bits are ACC
		UInt8* discKeyBlock; // must be 2048 bytes
		UInt8* titleKeyBlock; // must b e 2048 bytes
	};

	// DiscTagProperties passed to SetProperty(Prop_DDP_DiscTag)
	// This is for using the DiscTag to aid in preserving the integrity of the
	// DDP/CMF file set throughout the process from authoring to replication.
	struct DiscTagProperties
	{
		AS_String owner;      // typically the studio or publisher (can be blank)
		AS_String titleName;  // the name of the title as specified by the publisher
		AS_String catalog;    // unique reference identifier as specified by the publisher to reference the job
		AS_String customer;   // may be the same as the owner or different if ordered by 3rd party (can be blank)
		AS_String site;       // site licensee from product dongle to identify author, loader, or builder of layer files
		AS_String jobRelated; // any job related data you wish to include
		DateTime expiration;  // expiration date of content as specified by publisher (can be blank).
	};

	// Specifies type of codec. Used in struct PlayingTimeInfo
	typedef UInt32 CodecType;
	static const CodecType Codec_MPEG2            = 0x00;
	static const CodecType Codec_AVC              = 0x01;
	static const CodecType Codec_VC1              = 0x02;
	static const CodecType Codec_LPCM             = 0x03;
	static const CodecType Codec_AC3              = 0x04;
	static const CodecType Codec_DTS              = 0x05;
	static const CodecType Codec_DolbyLossless    = 0x06;
	static const CodecType Codec_DolbyDigitalPlus = 0x07;
	static const CodecType Codec_DTS_HD           = 0x08;
	static const CodecType Codec_DTS_HD_XLL       = 0x09;

	// PlayingTimeInfo passed to SetProperty(Prop_BDCMF_PlayingTimeInfo)
	// This indicates the duration of time used for the codec indicated
	// This property should be set for each codec used
	struct PlayingTimeInfo
	{
		CodecType codec; // type of codec
		UInt32 hours;    // hours
		UInt32 minutes;  // minutes
		UInt32 seconds;  // seconds
		UInt32 mseconds; // milliseconds
	};

	// CpsUnitInfo passed to SetProperty(Prop_BDCMF_CpsUnitInfo)
	// Indicates the CPS unit number and encryption usage for each
	// m2ts file on a BDCMF volume.
	// Client should set one CpsUnitInfo for each m2ts file added
	struct CpsUnitInfo
	{
		AS_String fileName; // full path of m2ts file on destination volume
		UInt32 cpsUnitNumber; // CPS unit number corresponding to this file
		bool encrypted; // indicates whether this file shall be encrypted on final disc
	};

	// CD-Text language support. Used as ExtendedInfo (see AS_Format_(Get/Set)Property)
	typedef UInt32 CDTextLanguageType;
	static const CDTextLanguageType CDTextLangGerman   = 0x08;
	static const CDTextLanguageType CDTextLangEnglish  = 0x09;
	static const CDTextLanguageType CDTextLangSpanish  = 0x0a;
	static const CDTextLanguageType CDTextLangFrench   = 0x0f;
	static const CDTextLanguageType CDTextLangItalian  = 0x15;
	static const CDTextLanguageType CDTextLangDutch    = 0x1d;
	static const CDTextLanguageType CDTextLangRussian  = 0x56;
	static const CDTextLanguageType CDTextLangKorean   = 0x65;
	static const CDTextLanguageType CDTextLangJapanese = 0x69;
	static const CDTextLanguageType CDTextLangChinese  = 0x75;

	// Supported destination protection types
	typedef UInt32 MDProtectionTypes;
	static const MDProtectionTypes MD_TargetProtection_CSS		   = (1<<1);	// Target protection only

	// Supported source protection types
	typedef UInt32 MDSrcProtectionTypes;
	static const MDSrcProtectionTypes MD_SrcProtection_ISO_SecureData = (1<<8);	// Source protection only
};


#ifdef __cplusplus
extern "C"
{
#endif

//
// Basic Layout API functions
//

//-----------------------------------------------------------------------------
// AS_StorageDevice_OpenFormat
//
// DESCRIPTION:
// Opens an AS_Format object, creating a data structure in RAM that is ready
// to receive file structure information from the media in the specified
// device. The volume's logical format (see AS_Volume_FS_Type) is output with
// the parameter kind (if the media is blank the kind will be
// AS_Volume::FS_None). A reference (AS_Format struct) is output with format,
// which may be used to refer to the format in subsequent calls. AS_Format
// inherits from AS_Volume, so it can also be used in functions that take an
// AS_Volume IN parameter.
//
// A callback with which the AuthorScript engine will report on the progress of
// the operation is specified with callback. The parameter userData allows the
// host application to specify a pointer to a memory location that will be passed
// back whenever AuthorScript calls the callback.
//
// NOTES:
// An error (AS_Error_NotSupported = 38) will result if the format of the volume
// is not supported by your version of AuthorScript Imaging. Please contact your
// Sonic representative if you have any questions about which formats are covered.
//
// RETURN:
//  AS_StorageError_None             - ok
//  AS_StorageError_InvalidParameter - if the flags are not defined, or other parameter contains invalid value
//  AS_StorageError_DeviceNotReady   - if there is no media recognized in device
//
AS_StorageError AS_API AS_StorageDevice_OpenFormat(
	const AS_StorageDevice& device,     // IN:  A reference to the AS_StorageDevice struct specifying the device containing the volume to open.
	AS_Volume::FS_Type& kind,           // OUT: A reference to the AS_Volume::FS_Type object that is to be filled in with the kind of volume detected.
	AS_Volume::InfoCallback callback,   // IN:  progress callback function
	void* userData,                    // IN:  progress callback userData
	AS_Format& format);                 // OUT: AS_Format reference

//-----------------------------------------------------------------------------
// AS_Format_Close
//
// DESCRIPTION:
// Releases the specified AS_Format object and all associated memory.
//
// NOTES:
// To update changes on media before releasing the AS_Format, first call AS_Volume_Flush.
//
// RETURN:
//  AS_StorageError_None             - ok
//  AS_StorageError_InvalidParameter - if the flags are not defined.
//
AS_StorageError AS_API AS_Format_Close(
	AS_Format& layout);                  // IN: layout reference

//-----------------------------------------------------------------------------
// AS_Format_Create
//
// DESCRIPTION:
// Mounts the specified AS_Format and formats it in the FS_Type (ISO, UDF 1.02,
// UDF 2.0, etc.) specified with the parameter fsKind, and the Format_Type
// (Type_VCD, Type_DVD, Type_BDMV, etc.) specified with the parameter
// formatKind. For some Format_Type's the fsKind parameter is ignored and the
// proper FS_Type for that Format_Type is used. A name for the newly-formatted
// AS_Format is specified with name. Limitations, if any, on writing to the
// formatted AS_Format are specified with access; for details see
// AS_Volume::AccessFlag.
//
// A callback with which the AuthorScript engine will report on the progress
// of the operation is specified with callback. The parameter userData allows
// the host application to specify a pointer to a memory location that will be
// passed back whenever AuthorScript calls the callback.
//
// This function will modify the media in preparation for writing in the following cases:
//  - for AS_Volume::Type_BDMV and AS_Volume::Type_BDAV: 
//     - blank DVD-R, DVD-R9, DVD+R and DVD+R9: reserves track for file system metadata 
//     - BDR: reserves tracks on media for file system metadata
//	   - blank, unformatted BDRE media: performs physical format of media
//
// NOTES:
//       An error will result if kind specifies an unsupported AS_Format type.
//       To determine if the media is blank before formatting, see AS_StorageDevice_GetDeviceProperty.
//
// RETURN:
//  AS_StorageError_None             - ok
//  AS_StorageError_InvalidParameter - if the flags are not defined, or other parameter contains an invalid value
//  AS_StorageError_InvalidAccess    - results if more than one flag is passed for AccessFlag
//  AS_StorageError_InvalidOperation - results if volume has already been mounted
//  AS_StorageError_Fatal            - if filesystem cannot be formatted
//
AS_StorageError AS_API AS_Format_Create(
	const AS_Format& format,                  // IN: reference to AS_Format handle
	const AS_Volume::FS_Type& fsKind,         // IN: kind of file system to write
	const AS_Volume::Format_Type& formatKind, // IN: kind of format desired
	const AS_File::Path & name,               // IN: Name to write as volume label
	const AS_Volume::AccessFlag & access,     // IN: must be either Mastering or RTTD
	AS_Volume::InfoCallback callback,         // IN: progress callback function
	void* userData);                         // IN: progress callback userData

//-----------------------------------------------------------------------------
// AS_Format_GetProperty
//
// DESCRIPTION:
// Retrieves the value of a property of the specified AS_Format. The parameter prop indicates the
// property (format, state, name, block size, free blocks, date modified, date created, etc.)
// for which to retrieve the current value. The value is output to a buffer pointed to with propData,
// the size of which is specified with bufLen. The actual size of the data written to the buffer
// is output with propSize.
//
// The extended optional parameters are provided for additional property data to be retrieved.
// If the extended info is a list, then the value of property prop for index PropertyIndex
// is provided in extpropData, the size of which is specified with extpropSize.
// If PropertyIndex = 0xffffffff, the whole list is written to extpropData.
//
// If extended info is not a list of items, then PropertyIndex value is ignored.
//
// NOTES:
//       Some properties require AS_Format to be formatted or mounted before being available to query their value.
//       Not all properties support additional property information.
//
// RETURN:
//  AS_StorageError_None             - ok
//  AS_StorageError_InvalidParameter - if the flags are not defined or if the property cannot be retrieved
//  AS_StorageError_BufferOverflow   - if the provided buffer length (bufLen) is less than required for this property
//  AS_StorageError_InvalidOperation - if the property cannot be obtained in the current volume state
//
AS_StorageError AS_API AS_Format_GetProperty(
	const AS_Format& format,         // IN: volume to get property on
	const AS_Format::Property& prop, // IN: which property to get (see AS_Format::Prop_...)
	UInt32 bufLen,                   // IN: size available in propData buffer
	void* propData,                  // OUT: buffer to copy property value into
	UInt32* propSize,                // OUT: size of data returned in propData
	UInt32 PropertyIndex=0,          // IN: index of list of property returned
	UInt32 extbufLen=0,              // IN: size available in extpropData buffer
	void*  extpropData=0,            // IN/OUT: extended buffer to copy property value into
	UInt32* extpropSize=0);          // OUT: size of data returned in extpropData

//-----------------------------------------------------------------------------
// AS_Format_SetProperty
//
// DESCRIPTION:
// Sets the value of a property of the specified AS_Format. The parameter prop indicates the property
// (e.g. name, date modified, date created, path delimiter) for which the value is to be set.
// The value itself is given in the buffer pointed to with propData, the size of which is specified with propSize.
//
// The extended optional parameters are provided for additional property data to be set.
// If the extended info is a list, then the value of property prop for index PropertyIndex
// is given in extpropData, the size of which is specified with extpropSize.
// If PropertyIndex = 0xffffffff, the whole list is set from extpropData.
//
// If extended info is not a list of items, then PropertyIndex value is ignored.
//
//
// NOTES:
//       All properties require volume to be formatted or mounted before being available to set their value.
//       Not all properties support additional property information.
//
// RETURN:
//  AS_StorageError_None             - ok
//  AS_StorageError_InvalidParameter - if the flags are not defined.
//  AS_StorageError_InvalidOperation - if the AS_Format has not been formatted/mounted
//  AS_StorageError_BufferOverflow   - if the provided buffer length (propSize) is less than required for this property
//  AS_StorageError_InvalidAccess    - if the AS_Format's AccessFlag was read-only
//
AS_StorageError AS_API AS_Format_SetProperty(
	const AS_Format& layout,          // IN: volume to set property on
	const AS_Format::Property& prop,  // IN: which property to set (see AS_Volume::Prop_...)
	UInt32 propSize,                  // IN: size of data in propData buffer
	void* propData,                   // IN: property data
	UInt32 PropertyIndex=0,           // IN:  index of list of property to set
	UInt32 extpropSize=0,             // IN: size available in extpropData buffer
	void*  extpropData=0);            // IN/OUT: extended buffer to copy property value into


//-----------------------------------------------------------------------------
// AS_Format_GetLayerBreakRange
//
// DESCRIPTION:
// Finds valid range of layer breaks for the specified AS_Format reference.
// Specifies the minimum and maximum valid layer breaks by filename and offset
// into the file.
// The values are to help the client determine what values can be used when
// setting the property AS_Format::Prop_LogicalLayerBrk.
//
// NOTES:
//       applies to Type_DVD and Type_HDDVD
//       should be called after adding all files, and before calling
//       AS_Volume_Prepare() or AS_Volume_Flush()
//
// RETURN:
//  AS_StorageError_None               - ok
//  AS_StorageError_InvalidParameter   - invalid AS_Format object passed in
//  AS_StorageError_InvalidOperation   - if the AS_Format has not been
//                                       formatted/mounted or the AS_Format is
//                                       not Type_DVD or Type_HDDVD
//  AS_StorageError_InvalidAccess      - if the AS_Format's AccessFlag was read-only
AS_StorageError AS_API AS_Format_GetLayerBreakRange(
	const AS_Format& format,            // IN: reference to AS_Format handle
	AS_File::Path& lowerLimitFilePath,  // OUT: full path of lower limit file
	UInt32& lowerLimitByteOffset,       // OUT: byte offset into lowerLimitFilePath
	AS_File::Path& upperLimitFilePath,  // OUT: full path of upper limit file
	UInt32& upperLimitByteOffset);      // OUT: byte offset into upperLimitFilePath

//
// Get/Set Layout XML
//

// Create a new Format from an XML Project (XML file provides target selection)
AS_StorageError AS_API AS_OpenFormatProject(
	const AS_PlanProject& project,            // IN: project text
	AS_Format& layout);                       // OUT: layout reference

// Set Properties for Format from an XML Project
AS_StorageError AS_API AS_Format_SetProjectProperties(
	const AS_Format& layout,                            // IN: layout reference
	const AS_PlanProject& projectInfo);                 // IN: project text

// Save current Format to an XML Project
AS_StorageError AS_API AS_Format_SaveProject(
	const AS_Format& layout,                   // IN: layout reference
	AS_PlanProject& projectInfo);        // IN: project text


#ifdef __cplusplus
}
#endif
#endif //_AS_FormatH
